indexing

	description: "Request language parser"
	author: "Andreas Leitner"
	date: "$Date: 2003/06/13 06:27:44 $"
	revision: "$Revision: 1.11 $"

deferred class TS_RT_PARSER

feature {NONE} -- Initialization

	make is
			-- Create new parser.
		do
		end

feature -- Status

	input: STRING
			-- Input string

	position: INTEGER
			-- Current position in input string

	has_error: BOOLEAN
			-- Has an error occured while parsing?

feature -- Status Setting

	set_input (an_input: like input) is
		require
			an_input_not_void: an_input /= Void
			an_input_not_empty: an_input.count > 0
		do
			input := an_input
			position := 1
			has_error := False
		ensure
			input_set: input = an_input
			position_initialized: position = 1
			no_error: not has_error
		end

	end_of_input: BOOLEAN is
			-- Has parsing reached end of input?
		require
			input: input /= Void
		do
			Result := position > input.count
		ensure
			definition: Result = (position > input.count)
		end

feature -- Access

	last_string: STRING

	last_integer: INTEGER

	last_object: ANY

	last_operand: TS_RT_OPERAND

feature -- Parsing

	parse_command_type is
			-- Parse command and make request available via `last_string'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			start_position: INTEGER
		do
			from
				start_position := position
			until
				end_of_input or else item = ':'
			loop
				forth
			end
			if end_of_input then
				has_error := True
			else
				if start_position + 1 = position then
					has_error := True
				else
					last_string := substring (start_position, position - 1)
				end
			end
			forth
		ensure
			no_error_implies_last_string: not has_error implies last_string /= Void
		end

	parse_reference is
			-- Parse a reference and make it available via `last_integer'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		do
			parse_token ("REFERENCE")
			if end_of_input then
				print ("error:syntax:didn't expect end of input%N")
			elseif not has_error then
				parse_integer
			end
		ensure
			no_error_implies_last_integer: not has_error implies last_integer > 0
		end

	parse_token (a_token: STRING) is
			-- Parse `a_token'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
			a_token_not_void: a_token /= Void
			a_token_not_empty: a_token.count > 0
		local
			i: INTEGER
		do
			from
				i := 1
			until
				end_of_input or else (i > a_token.count or has_error)
			loop
				if a_token.item (i) /= item then
					has_error := True
				else
					i := i + 1
					forth
				end
			end
			if i /= a_token.count + 1 then
				has_error := True
			end
		end

	parse_value is
			-- Parse value and make it available via `last_string'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			start_position: INTEGER
		do
			if item /= '"' then
				has_error := True
			else
				from
					forth
					start_position := position
				until
					end_of_input or else item = '"'
				loop
					forth
				end
				if end_of_input then
					has_error := True
				else
					last_string := substring (start_position, position - 1)
					forth
				end
			end
		end

	parse_integer is
			-- Parse integer and make it available via `last_integer'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		do
			parse_value
			if not has_error then
				if not last_string.is_integer then
					has_error := True
				else
					last_integer := last_string.to_integer
				end
			end
		end

	parse_identifier is
			-- Parse identifier and make it available via `last_string'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			start_position: INTEGER
		do
			if not is_letter (item) then
				has_error := True
			else
				from
					start_position := position
				until
					end_of_input or else not is_letter (item)
				loop
					forth
				end
				last_string := substring (start_position, position - 1)
			end
		end

	is_primitive_type_name (a_type_name: STRING): BOOLEAN is
			-- Is `a_type_name' the name of a primitive type?
		require
			a_type_name_not_void: a_type_name /= Void
			a_type_name_not_empty: a_type_name.count > 0
		do
			if
				a_type_name.is_equal ("BOOLEAN") or
				a_type_name.is_equal ("INTEGER") or
				a_type_name.is_equal ("INTEGER_8") or
				a_type_name.is_equal ("INTEGER_16") or
				a_type_name.is_equal ("INTEGER_64") or
				a_type_name.is_equal ("CHARACTER") or
				a_type_name.is_equal ("WIDE_CHARACTER") or
				a_type_name.is_equal ("DOUBLE") or
				a_type_name.is_equal ("REAL")
			then
				Result := True
			end
		end

	parse_manifest_constant (a_type_name: STRING; a_value: STRING) is
			-- Parse `a_value' given its type is `a_type_name' and make
			-- object available via `last_object'.
		require
			a_type_name_not_void: a_type_name /= Void
			a_value_not_void: a_value /= Void
		do
			last_object := Void
			if a_type_name.is_equal ("BOOLEAN") then
				if a_value.is_boolean then
					last_object := a_value.to_boolean
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("INTEGER") then
				if a_value.is_integer then
					last_object := a_value.to_integer
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("INTEGER_8") then
				if a_value.is_integer then
					last_object := a_value.to_integer.to_integer_8
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("INTEGER_16") then
				if a_value.is_integer then
					last_object := a_value.to_integer.to_integer_16
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("INTEGER_64") then
				if a_value.is_integer then
					last_object := a_value.to_integer_64
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("CHARACTER") then
				if a_value.count > 0 then
					last_object := a_value.item (1)
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("WIDE_CHARACTER") then
				if a_value.count > 0 then
					last_object := a_value.item (1)
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("DOUBLE") then
				if a_value.is_double then
					last_object := a_value.to_double
				else
					has_error := True
				end
			elseif a_type_name.is_equal ("REAL") then
				if a_value.is_real then
					last_object := a_value.to_real
				else
					has_error := True
				end
			end
		end

	parse_operand is
			-- Parse operand and make result available via `last_operand'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			type_name: STRING
			value: STRING
		do
			parse_identifier
			if has_error then
				-- has_error := True
			elseif end_of_input then
				has_error := True
			else
				type_name := last_string
				parse_value
				if has_error then
					-- has_error := True
				else
					value := last_string
					if type_name.is_equal ("REFERENCE") then
						if value.is_integer then
							create last_operand.make_reference (value.to_integer)
						else
							has_error := True
						end
					elseif is_primitive_type_name (type_name) then
						parse_manifest_constant (type_name, value)
						create last_operand.make_constant (type_name, last_object)
					else
						has_error := True
					end
				end
			end

		end

feature {NONE} -- Implementation

	forth is
		require
			input: input /= Void
		do
			position := position + 1
		ensure
			definition: position = old position + 1
		end

	item: CHARACTER is
			-- Character at current position
		require
			input: input /= Void
			not_end_of_input: not end_of_input
		do
			Result := input.item (position)
		ensure
			definition: Result = input.item (position)
		end

	substring (a_start_index, an_end_index: INTEGER): STRING is
			-- Substring of index from `a_start_index' to `an_end_index'
		require
			input: input /= Void
			valid_start_index: valid_position (a_start_index)
			valid_end_index: valid_position (an_end_index)
		do
			Result := input.substring (a_start_index, an_end_index)
		end

	valid_position (an_index: INTEGER): BOOLEAN is
		require
			input: input /= Void
		do
			Result := an_index > 0 and an_index <= input.count
		end

	is_letter (a_char: CHARACTER): BOOLEAN is
			-- Is `a_char' a letter ?
		do
			Result := True
			inspect a_char
			when 'A'..'Z' then
			when 'a'..'z' then
			when '_' then
			else
				Result := False
			end
		end

	multi_line_value_start_tag: STRING is "---multi-line-value-start---"
	multi_line_value_end_tag: STRING is "---multi-line-value-end---"

invariant

	valid_position: input /= Void implies (position >= 0 and position <= input.count + 1)

end
